>From 8a962b1e1f725648412158888499d97ccd17976b Mon Sep 17 00:00:00 2001
From: Joseph Tulou <brezhoneg1@yahoo.fr>
Date: Fri, 23 Jan 2009 23:46:53 +0100
Subject: [PATCH] implements persistent main window for qt4 ( --qt-keep-size )

---
 modules/gui/qt4/components/interface_widgets.cpp |    7 +++
 modules/gui/qt4/main_interface.cpp               |   51 ++++++++++++++++++++++
 modules/gui/qt4/main_interface.hpp               |    1 +
 modules/gui/qt4/qt4.cpp                          |    8 +++
 4 files changed, 67 insertions(+), 0 deletions(-)

diff --git a/modules/gui/qt4/components/interface_widgets.cpp b/modules/gui/qt4/components/interface_widgets.cpp
index 8f49308..2fb3956 100644
--- a/modules/gui/qt4/components/interface_widgets.cpp
+++ b/modules/gui/qt4/components/interface_widgets.cpp
@@ -104,6 +104,13 @@ void *VideoWidget::request( vout_thread_t *p_nvout, int *pi_x, int *pi_y,
                             unsigned int *pi_width, unsigned int *pi_height )
 {
     msg_Dbg( p_intf, "Video was requested %i, %i", *pi_x, *pi_y );
+
+    if( config_GetInt( p_intf, "qt-keep-size" ) )
+    {
+        *pi_width  = size().width();
+        *pi_height = size().height();
+    }
+
     emit askVideoWidgetToShow( *pi_width, *pi_height );
     if( p_vout )
     {
diff --git a/modules/gui/qt4/main_interface.cpp b/modules/gui/qt4/main_interface.cpp
index 980e8c6..f3a6a80 100644
--- a/modules/gui/qt4/main_interface.cpp
+++ b/modules/gui/qt4/main_interface.cpp
@@ -394,6 +394,15 @@ void MainInterface::handleMainUi( QSettings *settings )
     mainLayout->insertWidget( settings->value( "ToolbarPos", 0 ).toInt() ? 0: 3,
                               controls, 0, Qt::AlignBottom );
 
+    /* retrieve previously saved main window size if asked for */ 
+    if( config_GetInt( p_intf, "qt-keep-size" )
+       &&
+       ( i_visualmode == QT_ALWAYS_VIDEO_MODE ||
+         i_visualmode == QT_MINIMAL_MODE )
+      )
+    {
+        resize( settings->value( "MainWindow/mainSize", QSize( 400, 300 ) ).toSize() );
+    }
 
     /* Finish the sizing */
     main->updateGeometry();
@@ -515,6 +524,25 @@ int MainInterface::privacyDialog( QList<ConfigControl *> *controls )
 
 QSize MainInterface::sizeHint() const
 {
+    if( config_GetInt( p_intf, "qt-keep-size" ) )
+    {
+        switch( i_visualmode )
+        {
+            case QT_NORMAL_MODE : 
+
+                if( videoIsActive && videoWidget->isVisible() )
+                    return getSettings()->value( "MainWindow/mainSize", 
+                                                  QSize( 400, 300 ) ).toSize() ;
+                break;
+
+            case QT_ALWAYS_VIDEO_MODE :
+            case QT_MINIMAL_MODE :
+
+                return size();
+                break;
+        }
+    }
+
     int nwidth  = controls->sizeHint().width();
     int nheight = controls->isVisible() ?
                   controls->size().height()
@@ -1068,6 +1096,29 @@ void MainInterface::keyPressEvent( QKeyEvent *e )
         e->ignore();
 }
 
+void MainInterface::resizeEvent( QResizeEvent * event )
+{
+    if( config_GetInt( p_intf, "qt-keep-size" ) )
+    {
+        switch( i_visualmode )
+        {
+            case QT_NORMAL_MODE : 
+
+                /* mainSize is saved only if a video is running */
+                if( videoIsActive && videoWidget->isVisible() )
+                    getSettings()->setValue( "MainWindow/mainSize", size() );
+                break;
+
+            case QT_ALWAYS_VIDEO_MODE :
+            case QT_MINIMAL_MODE :
+
+                /* mainSize is saved without restriction */
+                getSettings()->setValue( "MainWindow/mainSize", size() );
+                break;
+        }
+    }
+}
+
 void MainInterface::wheelEvent( QWheelEvent *e )
 {
     int i_vlckey = qtWheelEventToVLCKey( e );
diff --git a/modules/gui/qt4/main_interface.hpp b/modules/gui/qt4/main_interface.hpp
index 2bb2d51..4b3f79f 100644
--- a/modules/gui/qt4/main_interface.hpp
+++ b/modules/gui/qt4/main_interface.hpp
@@ -142,6 +142,7 @@ private:
     virtual void customEvent( QEvent *);
     virtual void keyPressEvent( QKeyEvent *);
     virtual void wheelEvent( QWheelEvent * );
+    virtual void resizeEvent( QResizeEvent * event );
 
 public slots:
     void undockPlaylist();
diff --git a/modules/gui/qt4/qt4.cpp b/modules/gui/qt4/qt4.cpp
index 308ef1e..f140fbf 100644
--- a/modules/gui/qt4/qt4.cpp
+++ b/modules/gui/qt4/qt4.cpp
@@ -84,6 +84,12 @@ static void ShowDialog   ( intf_thread_t *, int, int, intf_dialog_args_t * );
 #define MINIMIZED_LONGTEXT N_( "VLC will start with just an icon in " \
                                "your taskbar" )
 
+#define KEEPSIZE_TEXT N_( "Confine videos to a predefined window size" )
+#define KEEPSIZE_LONGTEXT N_( "You can choose to confine a video into a " \
+                              "predefined window or let it expand to" \
+                              "to fit the original width/height/aspect-ratio. " \
+                              "By default, videos are run at original size." )
+
 #define TITLE_TEXT N_( "Show playing item name in window title" )
 #define TITLE_LONGTEXT N_( "Show the name of the song or video in the " \
                            "controler window title." )
@@ -176,6 +182,8 @@ vlc_module_begin ()
               SYSTRAY_LONGTEXT, false);
     add_bool( "qt-start-minimized", false, NULL, MINIMIZED_TEXT,
               MINIMIZED_LONGTEXT, true);
+    add_bool( "qt-keep-size", false, NULL, KEEPSIZE_TEXT,
+              KEEPSIZE_LONGTEXT, true )
     add_bool( "qt-name-in-title", true, NULL, TITLE_TEXT,
               TITLE_LONGTEXT, false );
     add_bool( "qt-fs-controller", true, NULL, QT_FULLSCREEN_TEXT,
-- 
1.5.2.5

